@@ -16,13 +16,17 @@ json_to_ts <- function(json_data) {
 long_to_ts <- function(data) {
   data_dt <- as.data.table(data)
   
-  data_dt[, date_zoo := as.numeric(as.yearmon(date)), by = series]
-  data_dt[is.na(date_zoo), date_zoo := as.numeric(as.yearqtr(date))]
+  data_dt[, `:=`(date_zoo = as.numeric(as.yearmon(date)), frq = 12), by = series]
+  data_dt[is.na(date_zoo), `:=`(date_zoo = as.numeric(as.yearqtr(date)), frq = 4)]
   
   dt_of_lists <- data_dt[, {
     dT <- diff(date_zoo)
     if(any(diff(dT) > 1e-6)) {
-      list(ts_object = list(xts(value, order_by = date_zoo)))
+      if(frq[1] == 4) {
+        list(ts_object = list(xts(value, order.by = as.yearqtr(date_zoo))))
+      } else {
+        list(ts_object = list(xts(value, order.by = as.yearmon(date_zoo))))
+      }
     } else {
       list(ts_object = list(ts(value, start = .SD[1, date_zoo], end = .SD[.N, date_zoo], deltat = dT[1])))
     }
@@ -35,7 +39,7 @@ long_to_ts <- function(data) {
   tslist
 }
 
-utils::globalVariables(c("date_zoo", "series", "ts_object", "value"))
+utils::globalVariables(c("date_zoo", "series", "ts_object", "value","frq"))
 
 #' @importFrom xts xts
 #' @importFrom zoo as.yearqtr as.yearmon
@@ -46,4 +50,4 @@ wide_to_ts <- function(data) {
   } else {
     long_to_ts(melt(data, id.vars = "date", variable.name = "series"))
   }
-}
\ No newline at end of file
+}
